/****************************************************************************
**
** Copyright (C) Datafox GmbH.
** All rights reserved.
** Contact: Datafox GmbH (develop@datafox.de)
**
** This file is part of the DFComDLL module of the Datafox SDK.
**
****************************************************************************/

#ifndef CHANNEL_4_BUFFER_H
#define CHANNEL_4_BUFFER_H

// defines fuer Setup
#include "file/interface/studio_setup_structs.h"

// Geraetedateiarchiv einbinden
#include "file/file_device_zip.h"

// Studiodatei einbinden
#include "file/file_aes.h"

// Datensatzgrosze fuer Zutrittslisten (Version 2) definieren
#define ENTRANCE2_RECORD_SIZE_0    33
#define ENTRANCE2_RECORD_SIZE_1    64
#define ENTRANCE2_RECORD_SIZE_2    20
#define ENTRANCE2_RECORD_SIZE_3    25
#define ENTRANCE2_RECORD_SIZE_4    21
#define ENTRANCE2_RECORD_SIZE_5    17
#define ENTRANCE2_RECORD_SIZE_6    21
#define ENTRANCE2_RECORD_SIZE_7    39

// forward declaration
class DocumentStudio;

class Channel4Buffer
{
public:
    Channel4Buffer();
    virtual ~Channel4Buffer();

    void ChangeOrderLong( uint8_t *Buf );
    uint32_t ChangeOrderULong( uint32_t val );
    void ULongToBinMSBFirst( uint8_t *Buf, uint32_t Val );
    void ChangeOrderInt( uint8_t *Buf );

public:
    uint32_t StartDatensatzB, EndDatensatzB;
    int MaxListB;
    int MaxDatB;

    StructRecordDescription DatensatzBeschreibung[MAX_RECORD_DESCRIPTION];
    StructRecordField DatensatzFeld[MAX_RECORD_DESCRIPTION][MAX_FIELDS];

    StructRecordDescription ListenBeschreibung[MAX_RECORD_DESCRIPTION];
    StructRecordField ListenFeld[MAX_RECORD_DESCRIPTION][MAX_FIELDS];

    uint32_t m_entrance2Pointer[MAX_RECORD_DESCRIPTION];
    StructTable2 m_entrance2Table[MAX_RECORD_DESCRIPTION];
    StructTable2Field m_entrance2TableField[MAX_RECORD_DESCRIPTION][MAX_FIELDS];

public:
    void reset();
    uint32_t GetMDEEKFlashSize_0400();
    uint32_t GetMDEEKFlashSize();
    uint32_t GetEntranceEKFlashSize();
    uint32_t GetEntrance2EKFlashSize();
    void AdjustExtendedMemPointer();
    void PrepareEntrance2EK( uint32_t offs, uint8_t *buf );
    uint32_t GetFlashSetupSize();


    // Speicher fuer Listendaten
    bool m_listSortCheck[MAX_RECORD_DESCRIPTION];
    uint32_t m_listSize[MAX_RECORD_DESCRIPTION];
    uint8_t m_ucListRecordSize[MAX_RECORD_DESCRIPTION];
    uint8_t *m_pucListMem[MAX_RECORD_DESCRIPTION];

    void ClearListBuffer();
    int MakeListe( int ListNum, int ListCnt, int ListSize, const uint8_t *Buf );

    // Sortieren der Datensaetze ueber ASCII - Feld. Offs ist Offset in Datensatz auf Feldbeginn.
    // Allgemein gültiger Sortieralgorithmus
    void QuicksortList( uint8_t *pucMem, uint8_t ucRecordSize, uint8_t ucOffs, int lLeftRecordNum, int lRightRecordNum );
    int QuicksortListPartition( uint8_t *pucMem, uint8_t ucRecordSize, uint8_t ucOffs, int lLeftRecordNum, int lRightRecordNum );
    void QuicksortListSwap( uint8_t *pucMem, uint8_t ucRecordSize, int lLeftRecordNum, int lRightRecordNum );

    static bool comparing( char *, char * );
    void SortList( uint8_t *pucMem, uint8_t ucRecordSize, uint8_t ucOffs );

    // Speicher fuer Zutrittskontrolle (Version 2) - Listendaten
    bool m_entrance2SortCheck[8];
    uint32_t m_ulEntrance2ListSize[8];
    uint8_t m_ucEntrance2RecordSize[8];
    uint8_t *m_pucEntrance2ListMem[8];

    void ClearEntrance2ListBuffer( int iListNum = -1 );

    int CheckEntrance2List( uint8_t num, uint16_t recordCount, const uint8_t *buf, uint32_t size );
    int CheckEntrance2Dec( const char *buf, int min, int max );
    int CheckEntrance2TM( const char *buf );
    int CheckEntrance2Date( const char *buf );
    int CheckEntrance2Time( const char *buf );
    int CheckEntrance2Weekday( const char *buf );
    int MakeEntrance2List( int ListNum, int ListCnt, int ListSize, const uint8_t *Buf );
    int ImportEntrance2List( uint8_t num, uint16_t recordCount, const uint8_t *buf, uint32_t size );

    // Speicher fuer Timeboy - Listendaten
    uint32_t m_ulTimeboyListSize[MAX_RECORD_DESCRIPTION][MAX_RECORD_DESCRIPTION];
    uint8_t *m_pucTimeboyListMem[MAX_RECORD_DESCRIPTION][MAX_RECORD_DESCRIPTION];
    int m_timeboyFlags[MAX_RECORD_DESCRIPTION][MAX_RECORD_DESCRIPTION];

    void ClearTimeboyListBuffer( uint8_t id = 0xFF, uint8_t num = 0xFF );
    int MakeTimeboyList( uint8_t id, uint8_t num, uint16_t recordCount, const uint8_t *buf, uint32_t size, int flags, int *error );

    bool WriteListToBinFile( uint8_t listNum, const char *filePath );
    bool WriteListToTextFile( uint8_t listNum, const char *filePath );
    bool WriteListEntrance2ToBinFile( uint8_t listNum, const char *filePath );
    bool WriteListEntrance2ToTextFile( uint8_t listNum, const char *filePath );
    bool WriteListTimeboyToBinFile( uint8_t listId, uint8_t listNum, const char *filePath );

    // fuer Setupdaten
    FileAES m_file;
    std::unique_ptr<DocumentStudio> m_doc;

    int GetTableCount( uint8_t ucList = 0 );

    int GetTableRecord( uint8_t ucNum, std::string &szName,
                        uint8_t *pucFieldCount, uint8_t *pucVariable = NULL,
                        uint8_t ucList = 0 );

    int GetTableField( uint8_t ucTableNum, uint8_t ucFieldNum,
                       std::string &szName, uint8_t *pucType,
                       uint8_t *pucLength, uint8_t ucList = 0 );

    void DefaultIOModule( StructIOModule *pIOModule );

    // Dateiarchiv fuer Geraetedaten (Firmware) verwalten
    FileDeviceZip m_fileDevice;

public:
    // Zutrittskontrolllisten (Version 2) aufsetzen.
    void Entrance2Default();
    int GetTableRecordEntrance2( uint8_t ucNum, std::string &szName,
                                 uint8_t *pucFieldCount );
    int GetTableFieldEntrance2( uint8_t ucTableNum, uint8_t ucFieldNum,
                                std::string &szName, uint8_t *pucType,
                                uint8_t *pucLength );
};

#endif
