#pragma once
#ifndef HARDWARE_INFO_STRUCTS_H
#define HARDWARE_INFO_STRUCTS_H

#include <cstdint>

#pragma pack(1)
///////////////////////////////////////
// Struct: StructHardwareInfo

/*!
    \brief Diese Struktur enthaelt die Informationen zum Geraet.
*/
struct StructHardwareInfo
{
    uint8_t version;
    uint8_t typeDevice;
    uint8_t typeBoard;
    uint8_t componentRS485_0;
    uint8_t typeTCPIP;
    uint8_t typeModemMC;
    uint8_t componentRS485_1;
    uint8_t typeTransponder;
    uint8_t typeDisplay;
    uint8_t typeKeypad;
    uint8_t typeMMC;
    uint8_t typeGPS;
    uint8_t componentUSB;
    uint8_t typeRadio;
    uint8_t typeDCF77;
    uint32_t componentInAnalog;
    uint32_t componentInDigital;
    uint32_t componentOutDigital;
    uint8_t typeFingerprint;
    uint8_t typeExternalReader;
    uint8_t optionIOModule;
    uint8_t optionTBAdaption;
    // Zutrittskontrolle 
#define HIAF_HAS_ZK       0x01 // Geraet nutzt Zutrittskontrolle

#define HIAF_HAS_OPTIONS2 0x02
    uint8_t administrationFlags;
    uint8_t optionComserver;
    uint8_t optionTPWrite;
    uint8_t optionHttp;
    uint8_t optionLanguage;
    uint8_t optionBiokeyUpdate;
#define HIAF_HAS_OPTIONS3 0x04
    uint16_t optionsSetup;
#define HIAF_HAS_OPTIONS4  0x08
    uint8_t typeTouch;
    uint8_t typeSensor;
    uint8_t componentBattery;
    uint8_t componentCharge;
#define HIAF_HAS_OPTIONS5 0x10
    uint8_t optionsPayable;

};

/*!
    \brief Headerstruktur für die gesamte Hardwareinfo.
*/ 
struct HARDWARECONFIGNODE
{
    uint8_t version;                   //!< Versionsnummer der Konfigurationsstrukturen
    uint8_t flags;                     //!< Flag-Feld fuer Konfigurationsuebergreifende Optionen.
                                       //!< Bits: Unused
    uint16_t device;                   //!< Geraetetypnummer. // -stimmt nicht mit dem Studio überein da einmal ein Zahlendreher gewesen ist
    uint32_t serial;                   //!< Seriennummer des Geraetes. // nicht genutzt vom HIP bzw. nur als fortlaufende Platinennummer
    uint16_t count;                    //!< Anzahl der folgenden Konfigurationsstrukturen.
                                       //!< Die Platznummer 0 ist fuer die Hauptplatine reserviert.
    uint16_t signature;                //!< Signatur der Hardware-Configuration.
    uint32_t signatureSize;            //!< Anzahl Byte Signaturrelevanter Daten.
    uint32_t ptrHardwareConfigArray;   //!< Offset auf das folgende Pointerarray der Offsets auf die einzelnen Folgestrukturen vom Typ HARDWARECONFIG.
                                       //!< Diese HARDWARECONFIG Strukturen können über eine HARDWARECONFIGSUBNODE Struktur über wiederum eigene HARDWARECONFIG's verfügen.
}; // 20 Byte

/*!
    \brief Struktur der Hardware-Konfiguration eines Moduls.
*/
struct HARDWARECONFIG
{
    uint16_t number;                        //!< Platznummer dieser Konfiguration.
    uint32_t id;                            //!< Typnummer des zugewiesenen Moduls.
    uint8_t revision;                       //!< Revision des Moduls. Bespiel 11 bedeutet Version 1.1
    uint8_t index;                          //!< Bestuecklistenindex des Moduls. Beispiel: 1 = a, 2 = b, 3 = c
    char article[11];                       //!< Interne Artikelnummer des Moduls.
    uint8_t type;                           //!< Type des Eintrags. Submodule oder Adapter etc. 0 = Mainboard, 1 = Modul, 2 = Modul mit Plätzen, 3 = Submodul, 4 = Erweiterungsplatine
                                            //!< Unterschied Modul und Submodul. Submodul kann auf ein Modul mit Plätzen bestückt werden
    char name[25];                          //!< Bezeichnung des Moduls. Zur Anzeige nutzbar.
    uint32_t ptrHardwareConfigSubNode;      //!< Offset auf die Folgestruktur vom Typ HARDWARECONFIGSUBNODE.
    uint8_t jumperCount;                    //!< Anzahl der zugeordneten Jumperstrukturen.
    uint32_t ptrHardwareConfigJumperArray;  //!< Offset auf das folgende Pointerarray der Offsets auf die einzelnen Strukturen vom Typ HARDWAREJUMPER.
}; // 54 Byte

/*!
    \brief Folgestruktur für Jumperdefinitionen der Platzteile.
*/
struct HARDWARECONFIGJUMPER
{
    uint8_t number;          //!< Nummer dieses Jumpers.
    uint8_t flags;           //!< Zusätliche Angaben.
                             //!< Bits: Unused
    uint8_t count;           //!< Anzahl der Pins.
    uint8_t start;           //!< Startpin einer Brücke.
    uint8_t end;             //!< Endpin einer Brücke.
    char name[25];           //!< Bezeichnung des Moduls. Zur Anzeige nutzbar.
}; //30 Byte

/*!
    \brief Folgestruktur der Module mit eigenen Plaetzen.
*/
struct HARDWARECONFIGSUBNODE
{
    uint8_t flags;                     //!< Flag-Feld fuer Konfigurationsuebergreifende Optionen.
                                       //!< Bits: Unused
    uint16_t count;                    //!< Anzahl der folgenden Konfigurationsstrukturen.
    uint32_t ptrHardwareConfigArray;   //!< Offset auf das folgende Pointerarray der Offsets auf die einzelnen Folgestrukturen vom Typ HARDWARECONFIG.
}; // 7 Byte

#pragma pack()

#endif
